@@ -261,7 +261,10 @@ def match(pattern, key, groups=False, partial=True):
         m = pop.match(kop, specials=True)
         if not m:
             return returns(None, [])
-        _matches.append(m.val)
+        if isinstance(m, (tuple, list)):
+            _matches.extend(_m.val for _m in m)
+        else:
+            _matches.append(m.val)
 
     # we've completed matching but the last item in match groups is treated 'greedily'
     assert kop is not None          # sanity
@@ -302,13 +305,11 @@ def assemble_multi(keys_list):
     >>> assemble_multi((['hello', 'there'], ['a', 1, 'c']))
     ('hello.there', 'a.1.c')
     """
-    out = ()
-    for keys in keys_list:
+    def _assemble(keys):
         keys = ([k] if isinstance(k, el.Op) else parse(quote(k)) for k in keys)
         iterable = itertools.chain.from_iterable(keys)
-        found = el.assemble(iterable)
-        out += (found,)
-    return out
+        return el.assemble(iterable)
+    return tuple(_assemble(keys) for keys in keys_list)
 
 
 def assemble(keys):
