@@ -193,7 +193,14 @@ class AppenderUnique(Appender):
 class FilterOp(Op):
     def is_pattern(self):
         return False
-    def is_filtered(self, node):
+
+    def filtered(self, items):
+        raise NotImplementedError
+
+    def matchable(self, op):
+        raise NotImplementedError
+
+    def match(self, op):
         raise NotImplementedError
 
 
@@ -221,13 +228,19 @@ class FilterKeyValue(FilterOp):
                 return False
         return True
 
-    def match_one(self, item):
-        if not isinstance(item, FilterKeyValue):
+    def filtered(self, items):
+        return (item for item in items if self.is_filtered(item))
+
+    def matchable(self, op):
+        return isinstance(op, FilterKeyValue)
+
+    def match(self, op):
+        if not self.matchable(op):
             return None
         r = ()
         for k, v in self.kv:
             found = False
-            for ik, iv in item.kv:
+            for ik, iv in op.kv:
                 if not k.matchable(ik) or not v.matchable(iv):
                     continue
                 mk = next(k.matches((ik.value,)), _marker)
@@ -239,25 +252,17 @@ class FilterKeyValue(FilterOp):
                 found = True
             if not found:
                 return None
-        return r
-
-    def matches(self, items):
-        r = ()
-        for item in items:
-            o = self.match_one(item)
-            if o is None:
-                return
-            if not o:
-                continue
-            r += (type(self)(*o),)
-        yield r
+        return type(op)(*r)
 
 
-def is_filtered(node, filters):
-    for f in filters:
-        if not f.is_filtered(node):
-            return False
-    return True
+class FilterKeyValueFirst(FilterKeyValue):
+    def matchable(self, op):
+        return isinstance(op, FilterKeyValueFirst)
+
+    def filtered(self, items):
+        for item in super().filtered(items):
+            yield item
+            break
 
 
 #
@@ -269,28 +274,54 @@ def itemof(node, val):
 
 
 class CmdOp(Op):
-    pass
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.filters = ()
+
+    def match(self, op):
+        results = ()
+        for f, of in zip(self.filters, op.filters):
+            if not f.matchable(of):
+                return None
+            m = f.match(of)
+            if m is None:
+                return None
+            results += (Match(m),)
+        return results
+
+    def filtered(self, items):
+        for f in self.filters:
+            items = f.filtered(items)
+        return items
 
 
 class Empty(CmdOp):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.filters = self.args
+
     def __repr__(self):
-        if self.args:
-            return '.'.join(repr(a) for a in self.args)
-        return ''
+        return '.'.join(repr(f) for f in self.filters)
+
     def is_pattern(self):
         return False
+
     def operator(self, top=False):
-        return ''
+        return self.__repr__()
+
     def values(self, node):
-        if is_filtered(node, self.args):
-            return (node,)
-        return ()
+        return self.filtered((node,))
+
     def default(self):
         return ''
-    def match(self, op, specials=False):
-        if isinstance(op, Empty):
-            return Match('')
-        return None
+
+    def match(self, op):
+        if not isinstance(op, Empty):
+            return None
+        m = super().match(op)
+        if m is None:
+            return m
+        return (Match(''),) + m
 
 
 class Key(CmdOp):
@@ -313,26 +344,39 @@ class Key(CmdOp):
         return '.'.join(repr(a) for a in self.args)
 
     def operator(self, top=False):
-        s = quote(self.op.value)
-        return s if top else '.' + s
+        iterable = itertools.chain((quote(self.op.value),), (repr(f) for f in self.filters))
+        s = '.'.join(iterable)
+        if top:
+            return s
+        return '.' + s
 
-    def is_filtered(self, node):
-        return is_filtered(node, self.filters)
+    def _items(self, node, keys):
+        curkey = None
+
+        def _values():
+            nonlocal curkey
+            for k in keys:
+                try:
+                    v = node[k]
+                except (TypeError, KeyError, IndexError):
+                    continue
+                curkey = k
+                yield v
+
+        def _items():
+            for v in self.filtered(_values()):
+                yield (curkey, v)
+
+        return _items()
 
     def items(self, node):
         if not hasattr(node, 'keys'):
-            return
-        keys = self.op.matches(node.keys()) if self.op is not None else node.keys()
-        for k in keys:
-            try:
-                v = node[k]
-            except (KeyError, TypeError, IndexError):
-                continue
-            if self.is_filtered(v):
-                yield (k, v)
+            return ()
+        return self._items(node, self.op.matches(node.keys()))
 
     def keys(self, node):
         return (k for k, _ in self.items(node))
+
     def values(self, node):
         return (v for _, v in self.items(node))
 
@@ -349,31 +393,18 @@ class Key(CmdOp):
     def match(self, op, specials=False):
         if not isinstance(op, Key):
             return None
-        results = ()
-        if self.op is None: # always matchable since None is implicit wilcard
-            if op.op is not None:
-                results += (Match(op.op.value),)
-        elif op.op is None:
-            if not isinstance(self.op, Wildcard):
-                return None
-        elif not self.op.matchable(op.op, specials):
+        if not self.op.matchable(op.op, specials):
             return None
-        else:
-            # match key
-            val = next(self.op.matches((op.op.value,)), _marker)
-            if val is _marker:
-                return None
-            results += (Match(val),)
 
-        # match filters
-        found = ()
-        for f in self.filters:
-            m = next(f.matches(op.filters), _marker)
-            if m is _marker:
-                return None
-            found += m
-        results += tuple(Match(f) for f in found)
+        results = super().match(op)
+        if results is None:
+            return None
 
+        # match key
+        val = next(self.op.matches((op.op.value,)), _marker)
+        if val is _marker:
+            return None
+        results += (Match(val),)
         return results
 
     def update(self, node, key, val):
@@ -404,18 +435,9 @@ class Slot(Key):
             return cls(Numeric(val))
         return String(val)
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        if isinstance(self.op, FilterOp):
-            self.filters = self.args
-            self.op = None
     def __repr__(self):
         return '[' + super().__repr__()  + ']'
 
-    def is_pattern(self):
-        # a pure filter treats op as an implicit wildcard
-        return super().is_pattern() or bool(self.filters)
-
     def operator(self, top=False):
         iterable = (repr(a) for a in self.filters)
         if self.op is not None:
@@ -424,22 +446,14 @@ class Slot(Key):
 
     def items(self, node):
         if hasattr(node, 'keys'):
-            yield from super().items(node)
-            return
+            return super().items(node)
 
-        if self.op is None:
-            keys = (idx for idx, _ in enumerate(node))
-        elif self.is_pattern():
+        if self.is_pattern():
             keys = self.op.matches(idx for idx, _ in enumerate(node))
         else:
             keys = (self.op.value,)
-        for k in keys:
-            try:
-                v = node[k]
-            except (KeyError, TypeError, IndexError):
-                continue
-            if self.is_filtered(v):
-                yield (k, v)
+
+        return self._items(node, keys)
 
     def default(self):
         if isinstance(self.op, Numeric) and self.op.is_int():
@@ -461,9 +475,6 @@ class Slot(Key):
         return node
 
     def upsert(self, node, val):
-        if hasattr(node, 'keys'):
-            if self.op is None:
-                raise RuntimeError('Bracket updates not permitted on dicts with naked op')
         return super().upsert(node, val)
 
     def pop(self, node, key):
@@ -532,6 +543,74 @@ class SlotSpecial(Slot):
         return node
 
 
+class SliceFilter(CmdOp):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.filters = self.args
+
+    def is_pattern(self):
+        return False
+
+    def match(self, op, specials=False):
+        if not isinstance(op, SliceFilter):
+            return None
+        return super().match(op)
+
+    def values(self, node):
+        return (type(node)(self.filtered(node)),)
+    def items(self, node):
+        return ((None, self.values(node)),)
+    def keys(self, node):
+        return (k for k, _ in self.items(node))
+
+    def upsert(self, node, val):
+        return self.update(node, None, val)
+
+    def update(self, node, key, val):
+        raise RuntimeError('Updates not supported for slice filtering')
+
+    def remove(self, node, val):
+        removes = []
+
+        curidx = None
+        def _items():
+            nonlocal curidx
+            for idx, item in enumerate(node):
+                curidx = idx
+                yield item
+
+        for v in self.filtered(_items()):
+            removes.append(curidx)
+
+        if not removes:
+            return node
+
+        def _build():
+            iterable = (v for idx, v in enumerate(node) if idx not in removes)
+            return type(node)(iterable)
+
+        # if we're removing by value, then we need to see _if_ new list will equal
+        new = None
+        if val is not ANY:
+            new = _build()
+            if new != val:
+                return node
+
+        # attempt to mutate
+        try:
+            for idx in reversed(removes):
+                del node[idx]
+            return node
+        except TypeError:
+            pass
+
+        # otherwise we can't mutate, so generate a new one
+        return _build() if new is not None else new
+
+    def pop(self, node, key):
+        return self.remove(node, ANY)
+
+
 class Slice(CmdOp):
     @classmethod
     def concrete(cls, val):
@@ -646,10 +725,10 @@ class Slice(CmdOp):
         return node.__class__(iterable)
     def remove(self, node, val):
         if val is ANY:
-            return self.pop(self, node, self.slice(node))
+            return self.pop(node, self.slice(node))
         key = self.slice(node)
         if node[key] == val:
-            return self.pop(self, node, key)
+            return self.pop(node, key)
         return node
 
 
